<?php

declare(strict_types=1);

namespace Drupal\sms\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the phone number verification entity.
 *
 * @ContentEntityType(
 *   id = "sms_phone_number_verification",
 *   label = @Translation("Phone Number Verification"),
 *   label_collection = @Translation("Phone Number Verifications"),
 *   label_singular = @Translation("phone number verification"),
 *   label_plural = @Translation("phone number verifications"),
 *   label_count = @PluralTranslation(
 *     singular = "@count phone number verification",
 *     plural = "@count phone number verifications",
 *   ),
 *   base_table = "sms_phone_number_verification",
 *   entity_keys = {
 *     "id" = "id",
 *   },
 * )
 */
class PhoneNumberVerification extends ContentEntityBase implements PhoneNumberVerificationInterface {

  /**
   * The bundle of the associated entity.
   *
   * This value is read on a database level.
   *
   * @var string
   */
  protected $bundle;

  public function getCreatedTime(): int {
    return (int) $this->get('created')->value;
  }

  public function getCreatedDate(): \DateTimeImmutable {
    return new \DateTimeImmutable('@' . (int) $this->get('created')->value);
  }

  public function getEntity(): ?EntityInterface {
    return $this->get('entity')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function setEntity(EntityInterface $entity) {
    $this->set('entity', $entity);
    return $this;
  }

  public function getPhoneNumber(): string {
    // @phpstan-ignore-next-line
    return (string) $this->get('phone')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setPhoneNumber(string $phone_number) {
    $this->set('phone', $phone_number);
    return $this;
  }

  public function getCode(): string {
    // @phpstan-ignore-next-line
    return (string) $this->get('code')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCode(string $code) {
    $this->set('code', $code);
    return $this;
  }

  public function getStatus(): bool {
    return (bool) $this->get('status')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setStatus(bool $status) {
    $this->set('status', $status);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type): array {
    $fields = [];

    $fields['id'] = BaseFieldDefinition::create('integer')
      ->setLabel(\t('Phone verification ID'))
      ->setDescription(\t('The phone verification ID.'))
      ->setReadOnly(TRUE)
      ->setSetting('unsigned', TRUE);

    $fields['entity'] = BaseFieldDefinition::create('dynamic_entity_reference')
      ->setLabel(\t('Entity'))
      ->setDescription(\t('The entity for this verification code.'))
      ->setRequired(TRUE)
      ->setReadOnly(TRUE);

    // Bundle field is required for statistics and bulk cleanup.
    $fields['bundle'] = BaseFieldDefinition::create('string')
      ->setLabel(\t('Bundle'))
      ->setDescription(\t('The bundle of the entity.'))
      ->setRequired(TRUE)
      ->setReadOnly(TRUE);

    $fields['phone'] = BaseFieldDefinition::create('telephone')
      ->setLabel(\t('Phone'))
      ->setDescription(\t('Phone number.'))
      ->setDefaultValue('')
      ->setCardinality(1)
      ->setDisplayOptions('form', [
        'type' => 'hidden',
      ]);

    $fields['code'] = BaseFieldDefinition::create('string')
      ->setLabel(\t('Verification code'))
      ->setDescription(\t('The generated verification code.'))
      ->setRequired(TRUE)
      ->setDefaultValue('');

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(\t('Created on'))
      ->setDescription(\t('The time that the verification code was created.'));

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(\t('Status'))
      ->setDescription(\t('Verification status. 0=not verified, 1=verified.'))
      ->setDefaultValue(FALSE)
      ->setRequired(TRUE);

    return $fields;
  }

  public function preSave(EntityStorageInterface $storage): void {
    parent::preSave($storage);
    // Update bundle field with bundle of entity.
    $entity = $this->getEntity();
    if ($entity instanceof EntityInterface) {
      $this->set('bundle', $entity->bundle());
    }
  }

}
