<?php

declare(strict_types=1);

namespace Drupal\sms\Message;

use Drupal\sms\Exception\SmsException;

/**
 * The result of an SMS messaging transaction.
 */
class SmsMessageResult implements SmsMessageResultInterface {

  /**
   * The error of the message, or NULL if unknown.
   *
   * @var \Drupal\sms\Message\SmsMessageResultStatus::*|null
   */
  protected ?string $error = NULL;

  /**
   * The error message as provided by the gateway API.
   *
   * @var string
   */
  protected string $errorMessage = '';

  /**
   * The message delivery reports.
   *
   * @var \Drupal\sms\Message\SmsDeliveryReportInterface[]
   */
  protected array $reports = [];

  /**
   * The credit balance after this message is sent, or NULL if unknown.
   *
   * This number is in the SMS gateway's chosen denomination.
   *
   * @var float|null
   */
  protected ?float $creditsBalance = NULL;

  /**
   * The credits consumed to process this message, or NULL if unknown.
   *
   * This number is in the SMS gateway's chosen denomination.
   *
   * @var float|null
   */
  protected ?float $creditsUsed = NULL;

  public function getError(): ?string {
    return $this->error;
  }

  /**
   * {@inheritdoc}
   */
  public function setError(?string $error) {
    if ($error !== NULL) {
      $reflection = new \ReflectionClass(SmsMessageResultStatus::class);
      if (\in_array($error, $reflection->getConstants(), TRUE) === FALSE) {
        throw new \LogicException(\sprintf('Invalid error passed to %s', __METHOD__));
      }
    }

    $this->error = $error;
    return $this;
  }

  public function getErrorMessage(): string {
    return $this->errorMessage;
  }

  /**
   * {@inheritdoc}
   */
  public function setErrorMessage($message) {
    $this->errorMessage = $message;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getReport($recipient): ?SmsDeliveryReportInterface {
    foreach ($this->reports as $report) {
      if ($report->getRecipient() == $recipient) {
        return $report;
      }
    }
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getReports(): array {
    return $this->reports;
  }

  /**
   * {@inheritdoc}
   */
  public function setReports(array $reports) {
    $this->reports = $reports;
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function addReport(SmsDeliveryReportInterface $report) {
    $this->reports[] = $report;
    return $this;
  }

  public function getCreditsBalance(): ?float {
    return $this->creditsBalance;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreditsBalance(?float $balance) {
    if (\is_numeric($balance) || \is_null($balance)) {
      $this->creditsBalance = $balance;
    }
    else {
      throw new SmsException(\sprintf('Credit balance set is a %s', \gettype($balance)));
    }
    return $this;
  }

  public function getCreditsUsed(): ?float {
    return $this->creditsUsed;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreditsUsed(?float $credits_used) {
    if (\is_numeric($credits_used) || \is_null($credits_used)) {
      $this->creditsUsed = $credits_used;
    }
    else {
      throw new SmsException(\sprintf('Credit used is a %s', \gettype($credits_used)));
    }
    return $this;
  }

}
