<?php

declare(strict_types=1);

namespace Drupal\sms\Provider;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Utility\Random;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\Entity\ConfigEntityStorageInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Utility\Token;
use Drupal\sms\Direction;
use Drupal\sms\Entity\PhoneNumberSettingsInterface;
use Drupal\sms\Entity\PhoneNumberVerificationInterface as EntityPhoneNumberVerificationInterface;
use Drupal\sms\Exception\PhoneNumberSettingsException;
use Drupal\sms\Message\SmsMessage;

/**
 * Phone number verification provider.
 */
class PhoneNumberVerification implements PhoneNumberVerificationInterface {

  /**
   * Constructs a new PhoneNumberVerification object.
   */
  public function __construct(
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly ConfigFactoryInterface $configFactory,
    private readonly Token $token,
    private readonly SmsProviderInterface $smsProvider,
    private readonly TimeInterface $time,
  ) {
  }

  public function getPhoneNumberSettings(string $entity_type_id, string $bundle): ?PhoneNumberSettingsInterface {
    /** @var \Drupal\sms\Entity\PhoneNumberSettingsInterface */
    return $this->phoneNumberSettings()
      ->load($entity_type_id . '.' . $bundle);
  }

  public function getPhoneNumberSettingsForEntity(EntityInterface $entity): PhoneNumberSettingsInterface {
    $phone_number_settings = $this->getPhoneNumberSettings($entity->getEntityTypeId(), $entity->bundle());
    if ($phone_number_settings === NULL) {
      throw new PhoneNumberSettingsException(\sprintf('Entity phone number config does not exist for bundle %s:%s', $entity->getEntityTypeId(), $entity->bundle()));
    }
    return $phone_number_settings;
  }

  /**
   * {@inheritdoc}
   */
  public function getPhoneVerificationByCode($code): false|EntityPhoneNumberVerificationInterface {
    /** @var \Drupal\sms\Entity\PhoneNumberVerificationInterface[] $entities */
    $entities = $this->phoneNumberVerificationStorage()
      ->loadByProperties([
        'code' => $code,
      ]);
    return \reset($entities);
  }

  /**
   * {@inheritdoc}
   */
  public function getPhoneVerificationByPhoneNumber(string $phone_number, ?bool $verified = TRUE, $entity_type = NULL): array {
    $properties = [];
    $properties['phone'] = $phone_number;
    if ($entity_type !== NULL) {
      $properties['entity__target_type'] = $entity_type;
    }
    if ($verified !== NULL) {
      $properties['status'] = (int) $verified;
    }

    /** @var \Drupal\sms\Entity\PhoneNumberVerificationInterface[] */
    return $this->phoneNumberVerificationStorage()
      ->loadByProperties($properties);
  }

  /**
   * {@inheritdoc}
   */
  public function getPhoneVerificationByEntity(EntityInterface $entity, $phone_number): ?EntityPhoneNumberVerificationInterface {
    /** @var \Drupal\sms\Entity\PhoneNumberVerificationInterface[] $entities */
    $entities = $this->phoneNumberVerificationStorage()
      ->loadByProperties([
        'entity__target_id' => $entity->id(),
        'entity__target_type' => $entity->getEntityTypeId(),
        'phone' => $phone_number,
      ]);
    return $entities !== [] ? $entities[\array_key_first($entities)] : NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function newPhoneVerification(EntityInterface $entity, $phone_number): ?EntityPhoneNumberVerificationInterface {
    $config = $this->getPhoneNumberSettingsForEntity($entity);
    $message = $config->getVerificationMessage() ?: '';

    // @todo Replace with code generator.
    $random = new Random();
    $code = \strtoupper($random->name(6));

    /** @var \Drupal\sms\Entity\PhoneNumberVerificationInterface $phone_verification */
    $phone_verification = $this->phoneNumberVerificationStorage()->create();
    $phone_verification
      ->setCode($code)
      ->setStatus(FALSE)
      ->setPhoneNumber($phone_number)
      ->setEntity($entity)
      ->save();

    $sms_message = new SmsMessage();
    $sms_message
      ->addRecipient($phone_number)
      ->setOption('_is_verification_message', TRUE)
      ->setMessage($message)
      ->setDirection(Direction::OUTGOING);

    $data = [];
    $data['sms-message'] = $sms_message;
    $data['sms_verification_code'] = $phone_verification->getCode();

    $sms_message
      ->setMessage($this->token->replace($message, $data))
      ->setAutomated(FALSE);

    $this->smsProvider
      ->queue($sms_message);

    return $phone_verification;
  }

  public function updatePhoneVerificationByEntity(EntityInterface $entity): void {
    try {
      $phone_number_settings = $this->getPhoneNumberSettingsForEntity($entity);
      $field_name = $phone_number_settings->getFieldName('phone_number');
      if ($field_name !== NULL) {
        $items_original = isset($entity->original) ? $entity->original->{$field_name} : NULL;
        $items = $entity->{$field_name};
      }
    }
    catch (PhoneNumberSettingsException $e) {
      // Missing phone number configuration for this entity.
    }

    // $items can be unassigned because field_name is not configured, or is NULL
    // because there is no items.
    if (!isset($items)) {
      return;
    }

    $numbers = [];
    \assert($items instanceof FieldItemListInterface);
    foreach ($items as $item) {
      $phone_number = $item->value;
      $numbers[] = $phone_number;

      if (NULL === $this->getPhoneVerificationByEntity($entity, $phone_number)) {
        $this->newPhoneVerification($entity, $phone_number);
      }
    }

    if (isset($items_original) && !$items->equals($items_original)) {
      foreach ($items_original as $item) {
        $phone_number = $item->value;
        // A phone number was deleted.
        if (!\in_array($phone_number, $numbers, TRUE)) {
          $phone_verification = $this->getPhoneVerificationByEntity($entity, $phone_number);
          if ($phone_verification !== NULL) {
            $phone_verification->delete();
          }
        }
      }
    }
  }

  public function deletePhoneVerificationByEntity(EntityInterface $entity): void {
    // Check the entity uses phone numbers. To save on a SQL call, and to
    // prevent having to install phone number verification for SMS Framework
    // tests which delete entities. Which would otherwise error on non-existent
    // tables.
    try {
      $this->getPhoneNumberSettingsForEntity($entity);
      $verification_entities = $this->phoneNumberVerificationStorage()
        ->loadByProperties([
          'entity__target_id' => $entity->id(),
          'entity__target_type' => $entity->getEntityTypeId(),
        ]);
      $this->phoneNumberVerificationStorage()->delete($verification_entities);
    }
    catch (PhoneNumberSettingsException) {
    }
  }

  public function purgeExpiredVerifications(): void {
    $current_time = $this->time->getRequestTime();

    $verification_ids = [];
    $prefix = 'sms.phone.';
    $ids = \array_map(
      static fn (string $id) => \substr($id, \strlen($prefix)),
      $this->configFactory->listAll($prefix),
    );

    foreach ($this->entityTypeManager->getStorage('phone_number_settings')->loadMultiple($ids) as $phoneNumberSetting) {
      \assert($phoneNumberSetting instanceof PhoneNumberSettingsInterface);
      $verification_ids += $this->phoneNumberVerificationStorage()->getQuery()
        ->accessCheck(FALSE)
        ->condition('entity__target_type', $phoneNumberSetting->getPhoneNumberEntityTypeId())
        ->condition('bundle', $phoneNumberSetting->getPhoneNumberBundle())
        ->condition('status', 0)
        ->condition('created', ($current_time - $phoneNumberSetting->getVerificationCodeLifetime()), '<')
        ->execute();
    }

    /** @var \Drupal\sms\Entity\PhoneNumberVerificationInterface $phone_number_verification */
    foreach ($this->phoneNumberVerificationStorage()->loadMultiple($verification_ids) as $phone_number_verification) {
      $entity = $phone_number_verification->getEntity();
      if ($entity !== NULL) {
        try {
          $config = $this->getPhoneNumberSettingsForEntity($entity);
          $purge = $config->getPurgeVerificationPhoneNumber();
          $field_name = $config->getFieldName('phone_number');
          if ($purge && $field_name !== NULL && $entity instanceof FieldableEntityInterface && $entity->hasField($field_name)) {
            $fieldList = $entity->get($field_name);
            $fieldList->filter(static function ($item) use ($phone_number_verification) {
              return $item->value !== $phone_number_verification->getPhoneNumber();
            });
            $entity->save();
          }
        }
        catch (EntityStorageException $e) {
          // Failed to save entity.
        }
      }
      $this->phoneNumberVerificationStorage()->delete([$phone_number_verification]);
    }
  }

  private function phoneNumberSettings(): ConfigEntityStorageInterface {
    /** @var \Drupal\Core\Config\Entity\ConfigEntityStorageInterface */
    return $this->entityTypeManager->getStorage('phone_number_settings');
  }

  private function phoneNumberVerificationStorage(): EntityStorageInterface {
    return $this->entityTypeManager->getStorage('sms_phone_number_verification');
  }

}
